import { defineComponent } from 'vue';
import * as common from 'schema-based-json-editor';
import { Select2 } from "select2-vue-component";
import { FileUploader } from "file-uploader-vue-component";
import { MarkdownTip } from "markdown-tip-vue";
import { Icon } from './icon';
import { Optional } from './optional';
import { Description } from './description';
import { stringEditorTemplateHtml } from './variables';
export var StringEditor = defineComponent({
    render: stringEditorTemplateHtml,
    components: {
        icon: Icon,
        optional: Optional,
        description: Description,
        select2: Select2,
        'file-uploader': FileUploader,
        'markdown-tip': MarkdownTip,
    },
    props: {
        schema: {
            type: Object,
            required: true,
        },
        initialValue: String,
        title: [String, Number],
        theme: {
            type: Object,
            required: true,
        },
        icon: {
            type: Object,
            requried: true,
        },
        locale: {
            type: Object,
            required: true,
        },
        readonly: Boolean,
        required: Boolean,
        hasDeleteButton: {
            type: Boolean,
            required: true,
        },
        noSelect2: {
            type: Boolean,
            required: true,
        },
        md: Object,
        hljs: Object,
        forceHttps: Boolean,
        monacoEditor: Object,
    },
    data: function () {
        return {
            value: '',
            errorMessage: '',
            buttonGroupStyle: common.buttonGroupStyleString,
            collapsed: false,
            imagePreviewStyle: common.imagePreviewStyleString,
            monacoCodeEditor: undefined,
        };
    },
    beforeMount: function () {
        this.value = common.getDefaultValue(this.required, this.schema, this.initialValue);
        this.validate();
        this.$emit('update-value', { value: this.value, isValid: !this.errorMessage });
    },
    mounted: function () {
        var _this = this;
        if (this.monacoEditor && this.$refs.monacoEditor) {
            this.monacoCodeEditor = this.monacoEditor.create(this.$refs.monacoEditor, {
                value: this.value,
                language: 'json',
                minimap: { enabled: false },
                lineNumbers: 'off'
            });
            var timer_1;
            this.monacoCodeEditor.onDidChangeModelContent(function () {
                clearTimeout(timer_1);
                timer_1 = setTimeout(function () {
                    _this.value = _this.monacoCodeEditor.getValue();
                    _this.validate();
                    _this.$emit('update-value', { value: _this.value, isValid: !_this.errorMessage });
                }, 500);
            });
        }
    },
    beforeUnmount: function () {
        if (this.monacoCodeEditor) {
            this.monacoCodeEditor.dispose();
        }
    },
    methods: {
        onChange: function (e) {
            this.value = e.target.value;
            this.validate();
            this.$emit('update-value', { value: this.value, isValid: !this.errorMessage });
        },
        updateSelection: function (value) {
            this.value = value;
            this.validate();
            this.$emit('update-value', { value: this.value, isValid: !this.errorMessage });
        },
        toggleOptional: function () {
            this.value = common.toggleOptional(this.value, this.schema, this.initialValue);
            this.validate();
            this.$emit('update-value', { value: this.value, isValid: !this.errorMessage });
        },
        collapseOrExpand: function () {
            this.collapsed = !this.collapsed;
        },
        fileGot: function (file) {
            var _this = this;
            var reader = new FileReader();
            reader.readAsDataURL(file);
            reader.onload = function () {
                _this.value = reader.result;
                _this.validate();
                _this.$emit('update-value', { value: _this.value, isValid: !_this.errorMessage });
            };
            reader.onerror = function (error) {
                console.log(error);
            };
        },
        validate: function () {
            this.errorMessage = common.getErrorMessageOfString(this.value, this.schema, this.locale);
        },
    },
    computed: {
        canPreviewImage: function () {
            return common.isImageUrl(this.value) || common.isBase64Image(this.value);
        },
        canPreviewMarkdown: function () {
            return this.md && this.schema.format === 'markdown';
        },
        canPreviewCode: function () {
            return this.hljs && this.schema.format === 'code';
        },
        canPreview: function () {
            return (!!this.value) && (this.canPreviewImage || this.canPreviewMarkdown || this.canPreviewCode);
        },
        useTextArea: function () {
            return this.value !== undefined
                && (this.schema.enum === undefined || this.isReadOnly)
                && (this.schema.format === 'textarea' || this.schema.format === 'code' || this.schema.format === 'json' || this.schema.format === 'markdown');
        },
        useInput: function () {
            return this.value !== undefined
                && !this.collapsed
                && (this.schema.enum === undefined || this.isReadOnly)
                && (this.schema.format !== 'textarea' && this.schema.format !== 'code' && this.schema.format !== 'json' && this.schema.format !== 'markdown');
        },
        useSelect: function () {
            return this.value !== undefined && this.schema.enum !== undefined && !this.isReadOnly;
        },
        useSelect2Component: function () {
            return this.useSelect && !this.noSelect2 && this.schema.format !== 'select' && this.schema.format !== 'radiobox';
        },
        useSelectComponent: function () {
            return this.useSelect && (this.schema.format === 'select' || this.noSelect2);
        },
        useRadioBoxComponent: function () {
            return this.useSelect && this.schema.format === 'radiobox';
        },
        getImageUrl: function () {
            return this.forceHttps ? common.replaceProtocal(this.value) : this.value;
        },
        getMarkdown: function () {
            return this.md.render(this.value);
        },
        getCode: function () {
            return this.hljs.highlightAuto(this.value).value;
        },
        isReadOnly: function () {
            return this.readonly || this.schema.readonly;
        },
        hasDeleteButtonFunction: function () {
            return this.hasDeleteButton && !this.isReadOnly;
        },
        willPreviewImage: function () {
            return this.value && !this.collapsed && this.canPreviewImage;
        },
        willPreviewMarkdown: function () {
            return this.value && !this.collapsed && this.canPreviewMarkdown;
        },
        willPreviewCode: function () {
            return this.value && !this.collapsed && this.canPreviewCode;
        },
        titleToShow: function () {
            return common.getTitle(this.title, this.schema.title);
        },
        options: function () {
            return common.getOptions(this.schema);
        },
        canUpload: function () {
            return this.schema.format === 'base64';
        },
        className: function () {
            var rowClass = this.errorMessage ? this.theme.errorRow : this.theme.row;
            return this.schema.className ? rowClass + ' ' + this.schema.className : rowClass;
        },
    },
});
